import static org.junit.Assert.*;

import org.junit.Before;
import org.junit.Test;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;
import org.springframework.integration.channel.QueueChannel;
import org.springframework.integration.support.MessageBuilder;
import org.springframework.messaging.Message;

import com.springinteg.chaining.OrderPayload;

public class ChainingHandlersTest {
	ApplicationContext context = null;

	@Before
	public void setUp() {
		context = new ClassPathXmlApplicationContext("spring-integ-context.xml");
	}

	@Before
	public void destroy() {
		context = null;
	}

	@Test
	public void testOrderChaining() {
		OrderPayload order = context.getBean("orderPayload", OrderPayload.class);
		order.setContent("orderMsg");
		Message<OrderPayload> orderMsg = MessageBuilder.withPayload(order).build();
		QueueChannel orderSendChannel = context.getBean("senderChannel", QueueChannel.class);
		orderSendChannel.send(orderMsg);
		QueueChannel orderRecvChannel = context.getBean("receiverChannel", QueueChannel.class);
		Message<?> orderRecvMsg = (Message<?>) orderRecvChannel.receive(2000);
		assertEquals(orderRecvMsg.getPayload().toString(), order.getContent());
		assertEquals(orderRecvMsg.getHeaders().get("enriched").toString(), "enriched-Header");

	}

	@Test
	public void testOrderChainingNested() {
		OrderPayload order = context.getBean("orderPayload", OrderPayload.class);
		order.setContent("orderMsg");
		Message<OrderPayload> coffeeMsg = MessageBuilder.withPayload(order).build();
		QueueChannel orderSendChannel = context.getBean("senderChannelForNested", QueueChannel.class);
		orderSendChannel.send(coffeeMsg);
		QueueChannel orderRecvChannel = context.getBean("receiverChannelForNested", QueueChannel.class);
		Message<?> orderRecvMsg = (Message<?>) orderRecvChannel.receive(2000);

		long mainChannelAccessTime = Long.valueOf((String) orderRecvMsg.getHeaders().get("main-channel")).longValue();
		long nestedChannelAccessTime = Long.valueOf((String) orderRecvMsg.getHeaders().get("nested-channel"))
				.longValue();
		assertTrue("Main channel should be always called before nested",
				nestedChannelAccessTime > mainChannelAccessTime);

	}
}
